﻿using System;

namespace ComboBoxSelectionDoubleBuffer
{
    class DoubleBuffer<T> : NotifyPropertyChangedImpl
    {
        /// <summary>
        /// Creates double buffer with default value
        /// </summary>
        public DoubleBuffer()
        {
        }

        /// <summary>
        /// Creates double buffer with specified value
        /// </summary>
        public DoubleBuffer(T value)
        {
            Assign(value);
        }

        /// <summary>
        /// UI-facing side of the buffer. Typically of no interest to the rest of the application
        /// </summary>
        public T UIValue
        {
            get { return _UIValue; }

            set // do not assign to UIValue directly - the setter should be called only by the binding
            {
                _UIValue = value; 
                if (UIValueChanged != null) UIValueChanged(value); 
            }
        }
        private T _UIValue;

        /// <summary>
        /// Application facing side of the buffer
        /// </summary>
        /// <remarks>Use Assign() to assign values to this side</remarks>
        public T Value
        {
            get { return _Value; }
            private set
            {
                _Value = value;
                if (ValueChanged != null) ValueChanged(value);
            }
        }
        private T _Value;

        public event Action<T> UIValueChanged;
        public event Action<T> ValueChanged;

        /// <summary>
        /// Forces specific value into Value and UIValue
        /// </summary>
        public void Assign(T value)
        {
            Value = value;
            _UIValue = value;
            RaisePropertyChanged("UIValue");
        }

        public void ConfirmUIChange()
        {
            Value = UIValue;
        }

        public void CancelUIChange()
        {
            _UIValue = Value;
            RaisePropertyChanged("UIValue");
        }
    }
}
